/*
 * Copyright 2008-2009 Ben Boeckel <MathStuf@gmail.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Header include
#include "BadgeGroupModel.h"

// Sigmodr tree includes
#include "BadgeModel.h"

// Sigmod includes
#include <sigmod/Badge.h>
#include <sigmod/Game.h>

// KDE includes
#include <KMenu>

using namespace Sigmod;
using namespace Sigmodr::Tree;

BadgeGroupModel::BadgeGroupModel(BaseModel* parent, Game* game) :
        GroupModel(parent, game, "Badges")
{
    for (int i = 0; i < game->badgeCount(); ++i)
        addObject(game->badge(i));
}

QVariant BadgeGroupModel::data(const int role) const
{
    if (role == BaseModel::ContextMenuRole)
    {
        KMenu* menu = new KMenu;
        menu->addAction("&Add Badge", this, SLOT(addObject()));
        return QVariant::fromValue(menu);
    }
    return GroupModel::data(role);
}

bool BadgeGroupModel::setData(const QVariant& value, int role)
{
    if (role == BaseModel::XmlRole)
    {
        QString data = value.toString();
        if (!data.isEmpty())
        {
            QDomDocument xml;
            if (loadFromData(data, &xml) && (xml.doctype().name() == "Badge"))
            {
                addObject(qobject_cast<Game*>(m_object)->newBadge(xml.documentElement()));
                return true;
            }
        }
    }
    return false;
}

QString BadgeGroupModel::types() const
{
    return "Badge";
}

void BadgeGroupModel::addObject(Object* object)
{
    if (!object)
        object = qobject_cast<Game*>(m_object)->newBadge();
    if (object->className() == "Badge")
        addChild(new BadgeModel(this, qobject_cast<Badge*>(object)));
}

void BadgeGroupModel::deleteObject(BaseModel* model)
{
    const int index = find(model);
    if (0 <= index)
    {
        qobject_cast<Game*>(m_object)->deleteBadge(index);
        m_objects[index]->deleteLater();
        m_objects.removeAt(index);
        childRowChanged(index);
    }
}
