/*
 * Copyright 2008-2009 Ben Boeckel <MathStuf@gmail.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Header include
#include "MapModel.h"

// Sigmodr tree includes
#include "MapEffectGroupModel.h"
#include "MapTrainerGroupModel.h"
#include "MapWarpGroupModel.h"
#include "MapWildListGroupModel.h"

// Sigmodr widget includes
#include <sigmodr/widgets/MapUI.h>

// Sigmod includes
#include <sigmod/Map.h>

// KDE includes
#include <KMenu>

using namespace Sigmod;
using namespace Sigmodr::Widgets;
using namespace Sigmodr::Tree;

MapModel::MapModel(BaseModel* parent, Map* map) :
        GroupObjectModel(parent, map)
{
    setupData();
}

MapModel::~MapModel()
{
    clearData();
}

QVariant MapModel::data(int role) const
{
    if (role == Qt::DisplayRole)
        return qobject_cast<Map*>(m_object)->name();
    else if (role == BaseModel::XmlRole)
    {
        QDomDocument xml(m_object->className());
        xml.appendChild(m_object->save());
        return xml.toString();
    }
    else if (role == BaseModel::WidgetRole)
    {
        QWidget* widget = new MapUI(qobject_cast<Map*>(m_object), NULL);
        return QVariant::fromValue(widget);
    }
    else if (role == BaseModel::ContextMenuRole)
    {
        KMenu* menu = new KMenu;
        menu->addAction("&Delete Map", this, SLOT(deleteSelf()));
        return QVariant::fromValue(menu);
    }
    return GroupObjectModel::data(role);
}

bool MapModel::setData(const QVariant& value, int role)
{
    if (role == BaseModel::XmlRole)
    {
        QString data = value.toString();
        if (!data.isEmpty())
        {
            QDomDocument xml;
            if (loadFromData(data, &xml))
            {
                if (xml.doctype().name() == m_object->className())
                {
                    clearData();
                    m_object->load(xml.documentElement());
                    setupData();
                    return true;
                }
                foreach (BaseModel* model, m_objects)
                {
                    GroupModel* group = qobject_cast<GroupModel*>(model);
                    if (group && group->setData(value, role))
                        return true;
                }
            }
        }
    }
    return false;
}

QString MapModel::types() const
{
    return type().append(QString(";MapEffect;MapTrainer;MapWarp;MapWildList"));
}

void MapModel::addObject(Object* object)
{
    Q_UNUSED(object)
}

void MapModel::deleteObject(BaseModel* model)
{
    Q_UNUSED(model)
}

void MapModel::deleteSelf()
{
    qobject_cast<GroupModel*>(m_parent)->deleteObject(this);
}

void MapModel::setupData()
{
    Map* map = qobject_cast<Map*>(m_object);
    addChild(new MapEffectGroupModel(this, map));
    addChild(new MapTrainerGroupModel(this, map));
    addChild(new MapWarpGroupModel(this, map));
    addChild(new MapWildListGroupModel(this, map));
}
