/*
 * Copyright 2008-2009 Ben Boeckel <MathStuf@gmail.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Header include
#include "MapTrainerGroupModel.h"

// Sigmodr tree includes
#include "MapTrainerModel.h"

// Sigmod includes
#include <sigmod/Map.h>
#include <sigmod/MapTrainer.h>

// KDE includes
#include <KMenu>

using namespace Sigmod;
using namespace Sigmodr::Tree;

MapTrainerGroupModel::MapTrainerGroupModel(BaseModel* parent, Map* map) :
        GroupModel(parent, map, "Trainers")
{
    for (int i = 0; i < map->trainerCount(); ++i)
        addObject(map->trainer(i));
}

QVariant MapTrainerGroupModel::data(const int role) const
{
    if (role == BaseModel::ContextMenuRole)
    {
        KMenu* menu = new KMenu;
        menu->addAction("&Add Trainer", this, SLOT(addObject()));
        return QVariant::fromValue(menu);
    }
    return GroupModel::data(role);
}

bool MapTrainerGroupModel::setData(const QVariant& value, int role)
{
    if (role == BaseModel::XmlRole)
    {
        QString data = value.toString();
        if (!data.isEmpty())
        {
            QDomDocument xml;
            if (loadFromData(data, &xml) && (xml.doctype().name() == "MapTrainer"))
            {
                addObject(qobject_cast<Map*>(m_object)->newTrainer(xml.documentElement()));
                return true;
            }
        }
    }
    return false;
}

QString MapTrainerGroupModel::types() const
{
    return "MapTrainer";
}

void MapTrainerGroupModel::addObject(Object* object)
{
    if (!object)
        object = qobject_cast<Map*>(m_object)->newTrainer();
    if (object->className() == "MapTrainer")
        addChild(new MapTrainerModel(this, qobject_cast<MapTrainer*>(object)));
}

void MapTrainerGroupModel::deleteObject(BaseModel* model)
{
    const int index = find(model);
    if (0 <= index)
    {
        qobject_cast<Map*>(m_object)->deleteTrainer(index);
        m_objects[index]->deleteLater();
        m_objects.removeAt(index);
        childRowChanged(index);
    }
}
