/*
 * Copyright 2008-2009 Ben Boeckel <MathStuf@gmail.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Header include
#include "MapTrainerModel.h"

// Sigmodr tree includes
#include "MapTrainerTeamMemberModel.h"
#include "GroupModel.h"

// Sigmodr widget includes
#include <sigmodr/widgets/MapTrainerUI.h>

// Sigmod includes
#include <sigmod/Map.h>
#include <sigmod/MapTrainer.h>
#include <sigmod/MapTrainerTeamMember.h>

// KDE includes
#include <KMenu>

using namespace Sigmod;
using namespace Sigmodr::Widgets;
using namespace Sigmodr::Tree;

MapTrainerModel::MapTrainerModel(BaseModel* parent, MapTrainer* trainer) :
        GroupObjectModel(parent, trainer)
{
    setupData();
}

MapTrainerModel::~MapTrainerModel()
{
    clearData();
}

QVariant MapTrainerModel::data(int role) const
{
    if (role == Qt::DisplayRole)
        return qobject_cast<MapTrainer*>(m_object)->name();
    else if (role == BaseModel::XmlRole)
    {
        QDomDocument xml(m_object->className());
        xml.appendChild(m_object->save());
        return xml.toString();
    }
    else if (role == BaseModel::WidgetRole)
    {
        QWidget* widget = new MapTrainerUI(qobject_cast<MapTrainer*>(m_object), NULL);
        return QVariant::fromValue(widget);
    }
    else if (role == BaseModel::ContextMenuRole)
    {
        KMenu* menu = new KMenu;
        menu->addAction("&Add Team Member", this, SLOT(addObject()));
        menu->addSeparator();
        menu->addAction("&Delete Trainer", this, SLOT(deleteSelf()));
        return QVariant::fromValue(menu);
    }
    return GroupObjectModel::data(role);
}

bool MapTrainerModel::setData(const QVariant& value, int role)
{
    if (role == BaseModel::XmlRole)
    {
        QString data = value.toString();
        if (!data.isEmpty())
        {
            QDomDocument xml;
            if (loadFromData(data, &xml))
            {
                if (xml.doctype().name() == m_object->className())
                {
                    clearData();
                    m_object->load(xml.documentElement());
                    setupData();
                    return true;
                }
                else if (xml.doctype().name() == "MapTrainerTeamMember")
                {
                    addObject(qobject_cast<MapTrainer*>(m_object)->newTeamMember(xml.documentElement()));
                    return true;
                }
            }
        }
    }
    return false;
}

QString MapTrainerModel::types() const
{
    return type().append(QString(";MapTrainerTeamMember"));
}

void MapTrainerModel::addObject(Object* object)
{
    if (!object)
        object = qobject_cast<MapTrainer*>(m_object)->newTeamMember();
    if (object->className() == "MapTrainerTeamMember")
        addChild(new MapTrainerTeamMemberModel(this, qobject_cast<MapTrainerTeamMember*>(object)));
}

void MapTrainerModel::deleteObject(BaseModel* model)
{
    const int index = find(model);
    if (0 <= index)
    {
        qobject_cast<MapTrainer*>(m_object)->deleteTeamMember(index);
        m_objects[index]->deleteLater();
        m_objects.removeAt(index);
        childRowChanged(index);
    }
}

void MapTrainerModel::deleteSelf()
{
    qobject_cast<GroupModel*>(m_parent)->deleteObject(this);
}

void MapTrainerModel::setupData()
{
    MapTrainer* trainer = qobject_cast<MapTrainer*>(m_object);
    for (int i = 0; i < trainer->teamMemberCount(); ++i)
        addChild(new MapTrainerTeamMemberModel(this, trainer->teamMember(i)));
}
