/*
 * Copyright 2008-2009 Ben Boeckel <MathStuf@gmail.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Header include
#include "SpeciesMoveModel.h"

// Sigmodr tree includes
#include "GroupModel.h"

// Sigmodr widget includes
#include <sigmodr/widgets/SpeciesMoveUI.h>

// Sigmod includes
#include <sigmod/Game.h>
#include <sigmod/Move.h>
#include <sigmod/Species.h>
#include <sigmod/SpeciesMove.h>

// KDE includes
#include <KMenu>

using namespace Sigmod;
using namespace Sigmodr::Widgets;
using namespace Sigmodr::Tree;

SpeciesMoveModel::SpeciesMoveModel(BaseModel* parent, SpeciesMove* move) :
        ObjectModel(parent, move)
{
}

QVariant SpeciesMoveModel::data(int role) const
{
    if (role == Qt::DisplayRole)
    {
        const Move* move = m_object->game()->move(qobject_cast<SpeciesMove*>(m_object)->move());
        if (move)
            return move->name();
        return "";
    }
    else if (role == BaseModel::XmlRole)
    {
        QDomDocument xml(m_object->className());
        xml.appendChild(m_object->save());
        return xml.toString();
    }
    else if (role == BaseModel::WidgetRole)
    {
        QWidget* widget = new SpeciesMoveUI(qobject_cast<SpeciesMove*>(m_object), NULL);
        return QVariant::fromValue(widget);
    }
    else if (role == BaseModel::ContextMenuRole)
    {
        KMenu* menu = new KMenu;
        menu->addAction("&Delete Move", this, SLOT(deleteSelf()));
        return QVariant::fromValue(menu);
    }
    return ObjectModel::data(role);
}

void SpeciesMoveModel::deleteSelf()
{
    qobject_cast<GroupModel*>(m_parent)->deleteObject(this);
}
