/*
 * Copyright 2008-2009 Ben Boeckel <MathStuf@gmail.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Header include
#include "TileModel.h"

// Sigmodr tree includes
#include "GroupModel.h"

// Sigmodr widget includes
#include <sigmodr/widgets/TileUI.h>

// Sigmod includes
#include <sigmod/Game.h>
#include <sigmod/Tile.h>

// KDE includes
#include <KMenu>

using namespace Sigmod;
using namespace Sigmodr::Widgets;
using namespace Sigmodr::Tree;

TileModel::TileModel(BaseModel* parent, Tile* tile) :
        ObjectModel(parent, tile)
{
}

QVariant TileModel::data(int role) const
{
    if (role == Qt::DisplayRole)
        return qobject_cast<Tile*>(m_object)->name();
    else if (role == BaseModel::XmlRole)
    {
        QDomDocument xml(m_object->className());
        xml.appendChild(m_object->save());
        return xml.toString();
    }
    else if (role == BaseModel::WidgetRole)
    {
        QWidget* widget = new TileUI(qobject_cast<Tile*>(m_object), NULL);
        return QVariant::fromValue(widget);
    }
    else if (role == BaseModel::ContextMenuRole)
    {
        KMenu* menu = new KMenu;
        menu->addAction("&Delete Tile", this, SLOT(deleteSelf()));
        return QVariant::fromValue(menu);
    }
    return ObjectModel::data(role);
}

void TileModel::deleteSelf()
{
    qobject_cast<GroupModel*>(m_parent)->deleteObject(this);
}
